
/**
 * @(#)Message.java	1.3 99/01/13
 *
 * Copyright (c) 1996-1999 James Begole.  All Rights Reserved.
 *
 * James Begole MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY 
 * OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. James Begole SHALL NOT BE LIABLE
 * FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * THIS SOFTWARE IS NOT DESIGNED OR INTENDED FOR USE OR RESALE AS ON-LINE
 * CONTROL EQUIPMENT IN HAZARDOUS ENVIRONMENTS REQUIRING FAIL-SAFE
 * PERFORMANCE, SUCH AS IN THE OPERATION OF NUCLEAR FACILITIES, AIRCRAFT
 * NAVIGATION OR COMMUNICATION SYSTEMS, AIR TRAFFIC CONTROL, DIRECT LIFE
 * SUPPORT MACHINES, OR WEAPONS SYSTEMS, IN WHICH THE FAILURE OF THE
 * SOFTWARE COULD LEAD DIRECTLY TO DEATH, PERSONAL INJURY, OR SEVERE
 * PHYSICAL OR ENVIRONMENTAL DAMAGE ("HIGH RISK ACTIVITIES"). James Begole
 * SPECIFICALLY DISCLAIMS ANY EXPRESS OR IMPLIED WARRANTY OF FITNESS FOR
 * HIGH RISK ACTIVITIES.
 *
 * @author   James "Bo" Begole
 * @author   Rich Burridge
 * @version  1.5 - 11th January 1999. 
 */

package examples.ppong;

import java.io.*;

class 
Message implements PpongDebugFlags {

    public static final char M_ACK       = '\u00A0';
    public static final char M_END       = '\u00A1';
    public static final char M_MISS      = '\u00A2';
    public static final char M_PAUSE     = '\u00A3';
    public static final char M_OPPONENT  = '\u00A4';
    public static final char M_RACKET    = '\u00A5';
    public static final char M_RESUME    = '\u00A6';
    public static final char M_RETARD    = '\u00A7';
    public static final char M_RETURN    = '\u00A8';
    public static final char M_SCORE     = '\u00A9';
    public static final char M_SERVER    = '\u00AA';
    public static final char M_START     = '\u00AB';
    public static final char M_TAKEBALL  = '\u00AC';
    public static final char M_TESTDELAY = '\u00AD';
    public static final char M_WALL      = '\u00AE';


    public static byte[]
    writeMessage(char type) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);

        try {
            dos.writeChar(type);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeMessage: exception: " + ioe);
        }
 
        return(baos.toByteArray());
    }


    public static byte[]
    writeOpponent(String name) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);

        try {
            dos.writeChar(M_OPPONENT);
            dos.writeUTF(name);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeOpponent: exception: " + ioe);
        }

        return(baos.toByteArray());
    }


    public static byte[]
    writeRacket(int x, int y) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);

	try {
	    dos.writeChar(M_RACKET);
	    dos.writeInt(x);
	    dos.writeInt(y);
	    dos.flush();
	} catch (IOException ioe) {
	    System.err.println("Message: writeRacket: exception: " + ioe);
	}

	return(baos.toByteArray());
    }


    public static byte[]
    writeRetard(boolean retard) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);
 
        try { 
            dos.writeChar(M_RETARD);
            dos.writeBoolean(retard);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeRetard: exception: " + ioe); 
        } 
 
        return(baos.toByteArray());
    }


    public static byte[]
    writeReturnTestDelay(long timeVal) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);
 
        try { 
            dos.writeChar(M_RETURN);
            dos.writeLong(timeVal);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeReturnTestDelay: exception: " + 
			       ioe); 
        } 
 
        return(baos.toByteArray());
    }


    public static byte[]
    writeScore(long gameNumber, String host1, int score1,
			        String host2, int score2) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);

        try {
            dos.writeChar(M_SCORE);
            dos.writeLong(gameNumber);
	    dos.writeUTF(host1);
            dos.writeInt(score1);
	    dos.writeUTF(host2);
            dos.writeInt(score2);
            dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeScore: exception: " + ioe);
        }

        return(baos.toByteArray());
    }


    public static byte[]
    writeServerMessage(String message) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);

        try {
            dos.writeChar(M_SERVER);
            dos.writeUTF(message);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeServerMessage: exception: " +
                               ioe);
        }

        return(baos.toByteArray());
    }


    public static byte[]
    writeStart(int XSpeed, int YSpeed, long gameNumber) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);
 
        try {
            dos.writeChar(M_START);
            dos.writeInt(XSpeed);
            dos.writeInt(YSpeed);
            dos.writeLong(gameNumber);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeStart: exception: " + ioe);
        }
 
        return(baos.toByteArray());
    }


    public static byte[]
    writeTakeBall(int x, int y, int xSpeed, int ySpeed) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);
 
        try { 
            dos.writeChar(M_TAKEBALL);
            dos.writeInt(x);
            dos.writeInt(y);
            dos.writeInt(xSpeed);
            dos.writeInt(ySpeed);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeTakeBall: exception: " + ioe); 
        } 
 
        return(baos.toByteArray());
    }


    public static byte[]
    writeTestDelay(long timeVal) {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        DataOutputStream      dos  = new DataOutputStream(baos);
 
        try {
            dos.writeChar(M_TESTDELAY);
            dos.writeLong(timeVal);
	    dos.flush();
        } catch (IOException ioe) {
            System.err.println("Message: writeTestDelay: exception: " + ioe);
        }

        return(baos.toByteArray());
    }
}
