
/*
 * @(#)AudioFileReader.java	1.12 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

/* Read from a Sun au audio file and produce AudioClick objects.
 *
 * @version     1.5 - 7th January 1999
 * @author	Behfar Razavi
 */

import java.io.RandomAccessFile;
import java.io.IOException;

public class
AudioFileReader implements SoundDebugFlags {

    String filename = null;
    int clickSize = 50;                /* Audio chunk size. */
    int magic;
    int sampleRate;                    /* Au file characteristics. */
    int fileLength; 
    int hdrSize;
    int channels;
    int encoding;
    int length = 0;                    /* Length of track in seconds. */
    RandomAccessFile audioFile;        /* Stream from the au file. */


    public
    AudioFileReader(String filename, int clickSize) throws IOException {
        if (AudioFileReader_Debug) {
            System.err.println("AudioFileReader: constructor: " +
                               " filename: " + filename +
			       " click size: " + clickSize);
        }

        this.filename  = filename;
	this.clickSize = clickSize;  /* Size of audio data to read. */

	getHeader(filename);
    }


    void
    getHeader(String filename) throws IOException {
        if (AudioFileReader_Debug) {
            System.err.println("AudioFileReader: getHeader: " +
                               " filename: " + filename);
        }

/* Open audiofile and read the header to determine sample rate, length, etc. */

        audioFile = new RandomAccessFile(filename, "r");

        magic      = audioFile.readInt();
        hdrSize    = audioFile.readInt();
        fileLength = audioFile.readInt();
        encoding   = audioFile.readInt();
        sampleRate = audioFile.readInt();
        channels   = audioFile.readInt();
                         
/* Determine the length of the audio track in seconds. Since the fileLength
 * field is optional, it may be set to ~0.  If so, deduce the length from
 * the length of audio file itself.
 */                      
                         
        if (fileLength <= 0) {
            length = (int) (audioFile.length() - hdrSize) /
                           (sampleRate * channels);
        } else {         
            length = fileLength / (sampleRate * channels);
        }                
        audioFile.seek(hdrSize);
    }


/*  Return the next chunk of audio data. Once the end of the file is
 *  reached, return an empty AudioClick to signal the end of the Media.
 */

    public AudioClick
    get() {
        int BUFSIZE = clickSize * sampleRate / 1000;   /* buffer size. */
        byte buf[]  = new byte[BUFSIZE];
        int n;

        if (AudioFileReader_Debug) {
            System.err.println("AudioFileReader: get.");
        }

        try {
            n = audioFile.read(buf, 0, BUFSIZE);
            if (n <= 0) {                         /* Reached end of file. */
                audioFile.close();
		getHeader(filename);
		n = audioFile.read(buf, 0, BUFSIZE);
	    }
        } catch (IOException e) {
	    return(new AudioClick(new byte[0], 0, 0));
        }

        return(new AudioClick(buf, 0, n));
    }


    public int
    getLength() {
        if (AudioFileReader_Debug) {
            System.err.println("AudioFileReader: getLength.");
        }

        return(length);
    }


    public int
    getSampleRate() {
        if (AudioFileReader_Debug) {
            System.err.println("AudioFileReader: getSampleRate.");
        }

        return(sampleRate);
    }


    public int
    getChannels() {
        if (AudioFileReader_Debug) {
            System.err.println("AudioFileReader: getChannels.");
        }

        return(channels);
    }


    public int
    getEncoding() {
        if (AudioFileReader_Debug) {
            System.err.println("AudioFileReader: getEncoding.");
        }

        return(encoding);
    }
}
