
/*
 * @(#)StockListener.java	1.9 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.stock;

import com.sun.media.jsdt.*;
import com.sun.media.jsdt.event.*;

/**
 * Stock quote listener. Uses yahoo to obtain stock quote information.
 *
 * @version     1.5 - 7th January 1999
 * @author      Amith Yamasani
 * @author      Rich Burridge
 */

public class
StockListener implements ByteArrayListener, StockDebugFlags {

    /** The name of the stock symbol to monitor. */
    String name;

    /** Set true if a new value has arrived. */
    boolean hasChanged = false;

    /** The new shared byte array value. */
    byte newValue[] = null;


    public
    StockListener(String name) {
        if (StockListener_Debug) {
            System.err.println("StockListener: constructor:" +
                               " name: " + name);
        }

        this.name = name;
    }


    public void
    byteArrayJoined(ByteArrayEvent event) {
        if (StockListener_Debug) {
            System.err.println("StockListener: byteArrayJoined:" +
                                " event: " + event);
        }
    }


    public void
    byteArrayLeft(ByteArrayEvent event) {
        if (StockListener_Debug) {
            System.err.println("StockListener: byteArrayLeft:" +
                                " event: " + event);
        }
    }


    public synchronized void
    byteArrayValueChanged(ByteArrayEvent event) {
	if (StockListener_Debug) {
            System.err.println("StockListener: byteArrayValueChanged: " +
                               " event: " + event);
        }

	try {
	    newValue = event.getByteArray().getValueAsBytes();
	} catch (NoSuchByteArrayException noe) {
            if (StockListener_Debug) {
                System.err.println("StockListener: byteArrayValueChanged:" +
                                   " exception " + noe);
                noe.printStackTrace();
            }
	}

	hasChanged = true;
    }


    public void
    byteArrayInvited(ByteArrayEvent event) {
        if (StockListener_Debug) {
            System.err.println("StockListener: byteArrayInvited:" +
                                " event: " + event);
        }
    }


    public void
    byteArrayExpelled(ByteArrayEvent event) {
        if (StockListener_Debug) {
            System.err.println("StockListener: byteArrayExpelled:" +
                                " event: " + event);
        }
    }
}
