
/*
 * @(#)StockPanel.java	1.23 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.stock;

import com.sun.media.jsdt.*;
import com.sun.media.jsdt.event.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.applet.Applet;
import java.net.*;

/**
 * Stock quote panel. Uses yahoo to obtain stock quote information.
 *
 * @version     1.5 - 7th January 1999
 * @author      Amith Yamasani
 * @author      Rich Burridge
 */

public class
StockPanel extends Panel implements StockDebugFlags, Runnable {

    /* States the the quotes text can be in. */
    static final int PLAY    = 0;
    static final int STOP    = 1;
    static final int REVERSE = 2;

    /** Current state of the quotes text. */
    private int state = PLAY;

    /** Are we running as an application (as opposed to an applet)? */
    private boolean isApplication = false;

    /** The name of the stock symbol to monitor. */
    private String name;

    /** The "last trade time" label in the stock title bar. */
    private Label timeLabel;

    /** The Shared byte array associated with this stock symbol. */
    private ByteArray sba;

    /** The byte array listener associated with this shared bytearray. */
    private StockListener listener;

    public StockButton play, stop, reverse;

    private boolean isValid = true;
    private Label symbol, value, change, quotes;

    private String quotesString = "   ";
    private String text = "   ";

    private int width  = 0;
    private int height = 0;

    Font         theFont = null;
    FontMetrics  fm      = null;
    Graphics     lg      = null;
    Graphics     imgG    = null;
    Image        image   = null;
    Thread       timer   = null;
    int          gPos = 0;
    int          pos  = 0;

    public
    StockPanel(Applet applet, Session session, Client client, 
	       Label timeLabel, String name, boolean isApplication) {
	Panel       p;
	StockButton button;
	URL         docBase = null;

	if (StockPanel_Debug) {
	    System.err.println("StockPanel: constructor: " +
				" applet: "     + applet +
				" session: "    + session +
				" client: "     + client +
				" time label: " + timeLabel +
				" name: "       + name +
				" application? " + isApplication);
	}

	this.name          = name;
	this.timeLabel     = timeLabel;
	this.isApplication = isApplication;

	if (isApplication == false) {
	    docBase = applet.getDocumentBase();
	}

	try {
	    sba = session.createByteArray(client, name, true);
	    sba.setValue(client, name.getBytes());
	    listener = new StockListener(name);
	    sba.addByteArrayListener(listener);
	} catch (JSDTException e) {
            if (StockPanel_Debug) {
                System.err.println("StockPanel: constructor: exception " + e);
                e.printStackTrace();
            }
	}

	setLayout(null);
	theFont = new Font("TimesRoman", Font.PLAIN, 14);
	setFont(theFont);
	setBackground(Color.black);
	setForeground(Color.yellow);

        symbol = new Label(name);               /* Stock symbol. */
	add(symbol);
	symbol.setSize(50, 25);
	symbol.setLocation(0, 0);

        value = new Label(" ");                 /* Last trade value. */
        add(value);
        value.setSize(100, 25);
        value.setLocation(50, 0);

        change = new Label(" ");                /* Change. */
        add(change);
        change.setSize(100, 25);
        change.setLocation(150, 0);

        timer = new Thread(this);

        play = new StockButton(this, "play", docBase, "images/play-on.gif", 
				 "images/play-off.gif");
	add(play);
	play.setSize(25, 25);
	play.setLocation(250, 0);

        stop = new StockButton(this, "stop", docBase, "images/stop-on.gif", 
				 "images/stop-off.gif");
	add(stop);
	stop.setSize(25, 25);
	stop.setLocation(275, 0);

        reverse = new StockButton(this, "reverse", docBase, 
				 "images/reverse-on.gif", 
				 "images/reverse-off.gif");
	add(reverse);
	reverse.setSize(25, 25);
	reverse.setLocation(300, 0);

	setQuoteState(PLAY);

	quotes = new Label(text) {
	    public void
            addNotify() {
		super.addNotify();
		timer.start();
	    }
        };                                     /* Quotes. */
        add(quotes);
        quotes.setSize(475, 25);
        quotes.setLocation(325, 0);
    }


    void
    removeListener() {
        if (StockPanel_Debug) {
            System.err.println("StockPanel: removeListener.");
        }

	try {
	    sba.removeByteArrayListener(listener);
        } catch (JSDTException e) {
            if (StockPanel_Debug) {
                System.err.println("StockPanel: constructor: exception " + e);
                e.printStackTrace();
            }
        }
    }


    void
    setQuoteState(int state) {
        if (StockPanel_Debug) {
            System.err.println("StockPanel: setQuoteState:" +
				" state: " + state);
        }

	this.state = state;
	play.setEngaged((state == PLAY) ? true : false);
	stop.setEngaged((state == STOP) ? true : false);
	reverse.setEngaged((state == REVERSE) ? true : false);
    }


    public void
    checkChange() {
	ByteArrayInputStream bais;
	DataInputStream      dis;

	if (listener.hasChanged == false) {
	    return;
	}

        if (StockPanel_Debug) {
            System.err.println("StockPanel: checkChange.");
        }

	bais = new ByteArrayInputStream(listener.newValue, 0, 
					listener.newValue.length);
	dis  = new DataInputStream(bais);

	try {
	    symbol.setText(dis.readUTF());          /* Stock symbol. */

	    isValid = dis.readBoolean();            /* Valid stock ticker? */

	    if (isValid) {
		timeLabel.setText(dis.readUTF());   /* Last trade time. */
	        value.setText(dis.readUTF());       /* Last trade value. */

	        change.setText(dis.readUTF());          /* Change. */
                if (change.getText().startsWith("-")) {
                    change.setForeground(Color.red);
                } else if (change.getText().startsWith("+")) {
                    change.setForeground(Color.green);
                } else {
                    change.setForeground(Color.yellow);
                }

	        quotesString  = "      ";
	        quotesString += dis.readUTF();     /* Quotes for this stock. */
	    } else {
		symbol.setForeground(Color.red);
		quotesString = " No such ticker symbol\n";
		scroll();
		setQuoteState(STOP);
	    }

	    listener.hasChanged = false;
	    if (StockPanel_Debug) {
                System.err.println("StockPanel: checkChange: " +
                                   " symbol: " + symbol +
				   " time: "   + timeLabel +
				   " value: "  + value +
				   " change: " + change +
				   " quotes: " + quotes);
            }
	} catch (IOException e) {
            if (StockPanel_Debug) {
                System.err.println("StockPanel: checkChange:" +
				   " exception " + e);
                e.printStackTrace();
            }
	}
    }


    public void
    run() {
        if (StockPanel_Debug) {
            System.err.println("StockPanel: run.");
        }

        while (true) {
            checkChange();
	    try {
                Thread.sleep(15);
            } catch (InterruptedException e) {
            }

	    if (state == PLAY) {
		scroll();
	    } else if (state == STOP) {
// Do nothing.
	    } else if (state == REVERSE) {
		reverseScroll();
	    }
        } 
    }


    public void
    resetQuote() {
        if (StockPanel_Debug) {
            System.err.println("StockPanel: resetQuote.");
        }

	pos = 0;
    }


    private synchronized void
    scroll() {
	int    currentWidth  = quotes.getSize().width;
        int    currentHeight = quotes.getSize().height;

        if (text == null || text.length() < 2) {
            return;
        }

        if (image == null || width != currentWidth || 
			     height != currentHeight) {
            image  = createImage(currentWidth, currentHeight);
            width  = currentWidth;
	    height = currentHeight;
	    imgG   = image.getGraphics();
            imgG.setFont(theFont);
        }
 
        imgG.setColor(Color.black);
        imgG.fillRect(0, 0, width, height);

        if (isValid) {
	    imgG.setColor(Color.yellow);
	} else {
	    imgG.setColor(Color.red);
	}

        if (fm == null) {
            fm = imgG.getFontMetrics();
        }
 
        gPos += 1;
        if (gPos >= fm.charWidth(text.charAt(pos))) {
            gPos = 0;
            pos++;
            if (pos >= text.length() - 1) {
                pos = 0;
                text = quotesString;
            }
        }
 
        int end = pos + 130;
        if (end > text.length() - 1) {
            end = text.length() - 1;
        }
        String newText = text.substring(pos, end);
 
        if (lg == null) {
            lg = quotes.getGraphics();
        }
        imgG.drawString(newText, -gPos, fm.getAscent());
        lg.drawImage(image, 0, 0, this);
    }


    private synchronized void
    reverseScroll() {
        int    currentWidth  = quotes.getSize().width;
        int    currentHeight = quotes.getSize().height;
 
        if (text == null || text.length() < 2) {
            return;
        }
 
        if (image == null || width != currentWidth || 
                             height != currentHeight) {
            image  = createImage(currentWidth, currentHeight);
            width  = currentWidth;
            height = currentHeight;
            imgG   = image.getGraphics();
            imgG.setFont(theFont);
        }
 
        imgG.setColor(Color.black);
        imgG.fillRect(0, 0, width, height);

        if (isValid) {
            imgG.setColor(Color.yellow);
        } else {
            imgG.setColor(Color.red);
        }

        if (fm == null) {
            fm = imgG.getFontMetrics();
        }
 
        gPos -= 1;
        if (gPos < 0) {
            pos--;
            if (pos <= 0) {
                text = quotesString;
                pos = text.length() - 1;
            }
            gPos = fm.charWidth(text.charAt(pos)) - 1;
        }

        int end = pos + 130;
        if (end > text.length() - 1) {
            end = text.length() - 1;
        }
        String newText = text.substring(pos, end);

        if (lg == null) {
            lg = quotes.getGraphics();
        }
        imgG.drawString(newText, -gPos, fm.getAscent());
        lg.drawImage(image, 0, 0, this);
    }
}


class 
StockButton extends Canvas implements MouseListener, StockDebugFlags {
    Image      upImage   = null;
    Image      downImage = null;

    boolean    engaged;
    String     label;
    StockPanel stockPanel;


    StockButton(StockPanel stockPanel, String label, URL docBase,
		String downImageName, String upImageName) {
        Toolkit toolkit = Toolkit.getDefaultToolkit();

        if (StockButton_Debug) {
            System.err.println("StockButton: constructor:" +
				" stock panel: "     + stockPanel +
				" label: "           + label +
				" url: "             + docBase +
				" down image name: " + downImageName +
				" up image name: "   + upImageName);
        }

	this.stockPanel = stockPanel;
	this.label      = label;

	if (docBase == null) {
	    downImage = toolkit.getImage(downImageName);
	    upImage   = toolkit.getImage(upImageName);
	} else {
	    try {
	        downImage = toolkit.getImage(new URL(docBase, downImageName));
	        upImage   = toolkit.getImage(new URL(docBase, upImageName));
            } catch (MalformedURLException e) {
            }
	}

	addMouseListener(this);
    }


    public void
    setEngaged(boolean engaged) {
        if (StockButton_Debug) {
            System.err.println("StockButton: setEngaged:" +
				" engaged: " + engaged);
        }

	this.engaged = engaged;
	repaint();
    }


    public void 
    paint(Graphics g) {
        if (StockButton_Debug) {
            System.err.println("StockButton: paint.");
        }

        update(g);
    }

 
    public void 
    update(Graphics g) {
        if (StockButton_Debug) {
            System.err.println("StockButton: update:" +
				" graphics: " + g);
        }

        if (engaged) {
            g.drawImage(downImage, 0, 0, this);
        } else {
            g.drawImage(upImage, 0, 0, this);
        }
    }

 
    public void 
    mouseClicked(MouseEvent e) {
    }


    public void 
    mousePressed(MouseEvent event) {
        if (StockButton_Debug) {
            System.err.println("StockButton: mousePressed:" +
                                " event: " + event);
        }

        repaint();
    }


    public void 
    mouseReleased(MouseEvent event) {
        int state = StockPanel.PLAY;

        if (StockButton_Debug) {
            System.err.println("StockButton: mouseUp:" +
                                " event: " + event);
        }

        if (label.equals("play"))  {
            state = StockPanel.PLAY;
        } else if (label.equals("stop")) {
            state = StockPanel.STOP;
        } else if (label.equals("reverse")) {
            state = StockPanel.REVERSE;
        }
     
        stockPanel.setQuoteState(state);
        repaint();
    }

 
    public void 
    mouseEntered(MouseEvent e) {
    }

 
    public void 
    mouseExited(MouseEvent e) {
    }
}
