
/*
 * @(#)StockServer.java	1.28 99/05/12
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.stock;

import com.sun.media.jsdt.*;
import com.sun.media.jsdt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Stock quote server. Uses yahoo to obtain stock quote information.
 *
 * @version     1.5 - 12th May 1999
 * @author      Amith Yamasani
 * @author      Rich Burridge
 */

public class
StockServer extends SessionAdaptor
	    implements ChannelConsumer, StockDebugFlags {

    private static final int QUOTE_INTERVAL = 10;    // Seconds.

    private static String sessionName = "StockSession";

    InputStreamReader   isr = null;
    BufferedReader      br  = null;

    StockClient stockClient  = null;
    Session     stockSession = null;
    Channel     stockChannel = null;

    Hashtable   stocks       = null;

    String      abort = null;
    String      tickers;

    int         noStocks = 0;
    int         timer;


    public
    StockServer(String args[]) {
	URLString   url          = null;
        String      sessionType  = null;
        String      hostname     = null;
        int         hostport     = 0;

        if (StockServer_Debug) {
            System.err.println("StockServer: constructor:" +
				" args: " + args);
        }

        setProps();

	stocks      = new Hashtable();
	hostname    = getHost(args);
        hostport    = getPort(args);
        sessionType = getType(args);
        url         = URLString.createSessionURL(hostname, hostport,
						 sessionType, sessionName);
 
        try {

/* Registry running?  Start it if it isn't. */

            if (RegistryFactory.registryExists(sessionType) == false) {
                RegistryFactory.startRegistry(sessionType);
            }

            stockClient  = new StockClient("Server");
	    stockSession = SessionFactory.createSession(stockClient,
							url, true);
	    stockSession.addSessionListener((SessionListener) this);

            stockChannel = stockSession.createChannel(stockClient, 
					"StockChannel", true, true, true);
	    stockChannel.addConsumer(stockClient, this);
        } catch (JSDTException e) {
            System.err.println("StockServer: constructor: exception: " + e);
            if (StockServer_Debug) {
                e.printStackTrace();
            }
        }
    }


    private void
    updateStockList(Session session, Client client) {
        StringBuffer buf     = new StringBuffer("");
	String       names[] = null;

	if (StockServer_Debug) {
            System.err.println("StockServer: updateStockList:" +
                                " session: " + session +
				" client: "  + client);
        }

	try {
	    names = session.listByteArrayNames();
	    if (names == null) {
		return;
	    }

	    for (int i = 0; i < names.length; i++) {
	        if (stocks.containsKey(names[i]) == false) {
		    ByteArray sba = session.createByteArray(client, 
							    names[i], true);
		    StockRecord record = new StockRecord(client, sba);

        	    if (StockServer_Debug) {
            	        System.err.println("StockServer: updateStockList:" +
					   " adding: " + names[i]);
		    }
		    stocks.put(names[i], record);
		    noStocks++;
	        }
	        buf.append(names[i]).append("+");
	    }
	} catch (JSDTException e) {
            System.err.println("StockServer: updateStockList: exception: " + e);
            if (StockServer_Debug) {
                e.printStackTrace();
            }
	}

	tickers = new String(buf);
    }


    private void
    run() {
        if (StockServer_Debug) {
            System.err.println("StockServer: run.");
        }

	try {
	    while (true) {
		updateURL();
		timer = QUOTE_INTERVAL;
		while (timer > 0) {
		    Thread.sleep(1000);
		    timer--;
		}
	    }
        } catch (Exception e) {
            System.err.println("StockServer: run: exception: " + e);
            if (StockServer_Debug) {
                e.printStackTrace();
            }
        }
    }
 
 
    private static String
    getHost(String args[]) {
        String defHost = "stard";  /* Default host name for connections. */
        int length = args.length;
 
        if (StockServer_Debug) {
            System.err.println("StockServer: getHost.");
        }
 
        for (int i = 0; i < length; i++) {
            if (args[i].equals("-server")) {
                if (++i < length) {
                    return(args[i]);
                }
            }    
        }
        return(defHost);
    }


    private static int
    getPort(String args[]) {
        int defPort = 4461;   /* Default port number for connections. */
        int length = args.length;

        if (StockServer_Debug) {
            System.err.println("StockServer: getPort.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-port")) {
                if (++i < length) {
                    return(Integer.parseInt(args[i]));
                }
            }    
        }
        return(defPort);
    }


    private static String
    getType(String args[]) {
        String defType = "socket";   /* Default Session type. */
        int length = args.length;

        if (StockServer_Debug) {
            System.err.println("StockServer: getType.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-type")) {
                if (++i < length) {
                    return(args[i]);
                }
            }    
        }
        return(defType);
    }


    private void 
    setProps() {
        Properties props = new Properties(System.getProperties());

        props = new Properties(props);

        File theUserPropertiesFile;
        String sep = File.separator;
        theUserPropertiesFile = new File(System.getProperty("user.home") +
                                         sep + ".hotjava" +
                                         sep + "properties");

        try {
            FileInputStream in = new FileInputStream(theUserPropertiesFile);
            props.load(new BufferedInputStream(in));
            in.close();
        } catch (Exception e) {
            System.err.println("StockServer: setProps: Error loading properties. Have you run hotjava or appletviewer before? Please do so, and set the firewall proxy in the preferences.");
        }
 
        System.setProperties(props);
    }


    private void 
    readURL(String sUrl) {
        if (StockServer_Debug) {
            System.err.println("StockServer: readURL:" +
				" url: " + sUrl);
        }

        try {
            if (StockServer_Debug) {
                System.err.println("StockServer: readURL: opening URL.");
            }

            URL url = new URL(sUrl);

            if (StockServer_Debug) {
                System.err.println("StockServer: readURL: reading content.");
            }

	    isr = new InputStreamReader((InputStream) url.getContent());
	    br  = new BufferedReader(isr);
        } catch (MalformedURLException e) {
            abort = "URL error";
        } catch (IOException ie) {
            abort = "IO error";
        }

        if (StockServer_Debug) {
            System.err.println("StockServer: readURL: done.");
        }
    }


    synchronized void 
    updateURL() {

        if (StockServer_Debug) {
            System.err.println("StockServer: updateURL.");
        }

        updateStockList(stockSession, stockClient);

/* If tickers is an empty string (ie: no stocks to monitor), just return. */

	if (tickers == null || tickers.length() == 0) {
	    if (StockServer_Debug) {
		System.err.println("StockServer: updateURL: " +
				   " no stocks to monitor at present.");
	    }
	    return;
	}

/* Read in the stock information again. */

        readURL("http://quote.yahoo.com/quotes?symbols=" + tickers + 
		"&detailed=f&options=t");
        parseReply();
    }


    private void
    parseReply() {
	int m, n;
	StockRecord current = null;
	String line, symbol, time, value, change, quote;
	boolean moreQuotes = true;

	if (StockServer_Debug) {
            System.err.println("StockServer: parseReply.");
        }

	try {
	    line = br.readLine();
	    if (line == null) {
		return;
	    }

	    while (line.startsWith("Symbol      Last Trade") == false) {
		line = br.readLine();
	    }

/* Found "Symbol      Last Trade..." line. */

	    for (int i = 0; i < noStocks; i++) {
		boolean valid = true;

		line = br.readLine();

/* If the stock line doesn't begin with a '<' character, we assume the stock     * doesn't exist.
 */

		valid = line.startsWith("<");
                if (valid == false) {                         // Symbol.
                    n = line.indexOf(" ");
		    if (n == -1) {
			System.err.println("Invalid line: [" + line + "].");
			continue;
		    }
		    symbol = line.substring(0, n);
		} else {
                    m = line.indexOf(">");
                    line = line.substring(m+1);
                    n = line.indexOf("<");
                    symbol = line.substring(0, n);
		}

                if (StockServer_Debug) {
                    System.err.println("UPDATE: Stock symbol: " + symbol);
                }

                current = (StockRecord) stocks.get(symbol);
                current.setValid(valid);

		if (valid == false) {
		    continue;
                }

		line = line.substring(n+1);                    // Time.
		m = line.indexOf(" ");
		time = line.substring(m, m+12).trim();
		current.setTime(time);

		if (StockServer_Debug) {
		    System.err.println("UPDATE: Last Trade time: " + time);
		}

		line = line.substring(m+12);
		m = line.indexOf(">");                         // Value.
		line = line.substring(m+1);
		n = line.indexOf("<");
		value = line.substring(0, n);
		current.setValue(value);

		if (StockServer_Debug) {
                    System.err.println("UPDATE: Last Trade value: " + value);
                }

		line = line.substring(n+1);                   // Change.
		m = line.indexOf("-");
                if (m == -1) {
		    m = line.indexOf("+");
		}
		if (m == -1) {
		    change = " ";
		} else {
		    line = line.substring(m);
		    for (n = 0; n < line.length(); n++) {
			if ((line.charAt(n) == ' ' && 
			     line.charAt(n+1) == ' ') ||
			     line.charAt(n) == '<') {
		            break;
			}
		    }
                    change = line.substring(0, n); 
                }
		current.setChange(change); 

                if (StockServer_Debug) {
                    System.err.println("UPDATE: Change: " + change);
                }
	    }

	    line = br.readLine();
            while (line.startsWith("<!-- Yahoo TimeStamp") == false) {
                line = br.readLine();
            }

/* Found "<!-- Yahoo TimeStamp..." line. Clear all the quotes fields from
 * the shared byte arrays.
 */

	    for (Enumeration e = stocks.elements(); e. hasMoreElements();) {
		StockRecord next = (StockRecord) e.nextElement();

		next.clearQuotes();
	    }

	    while (moreQuotes) {
		line = br.readLine(); 
                if (line.startsWith("</pre") ||
		    line.startsWith("<table")) {
		    moreQuotes = false;
		    continue;
		}

/* XXX: might be more than one company name associated with a quote. */

                m = line.indexOf(" ");                      // Symbol.
                m = line.indexOf(" ", m+1);
		m = line.indexOf(" ", m+1);
                line = line.substring(m+1);

		n = line.indexOf(" ");
                if (m == -1 || n == -1) {
		    continue;
                }

		symbol = line.substring(0, n);
                if (StockServer_Debug) {
                    System.err.println("QUOTES: Stock symbol: " + symbol);
                }

                if ((current = (StockRecord) stocks.get(symbol)) == null) {
		    continue;
		}

                m = line.indexOf("\">");
                n = line.indexOf("</a>");
                if (m == -1 || n == -1) {
		    continue;
		}

		quote = line.substring(m+2, n);
                if (StockServer_Debug) {
                    System.err.println("QUOTES: Quote: " + quote);
                }

		current.addQuote(quote);
            }

/* Update all the shared byte arrays. */

            for (Enumeration e = stocks.elements(); e. hasMoreElements();) {
                StockRecord next = (StockRecord) e.nextElement();
 
                next.update();
            }   
	} catch (IOException e) {
            System.err.println("StockServer: parseReply: exception: " + e);
            if (StockServer_Debug) {
                e.printStackTrace();
            }
	}

/* Found "</pre>" line. */

    }


    public String
    getName() {
        if (StockServer_Debug) {
            System.err.println("StockServer: getName.");
	}

	return("Server");
    }


    public void
    dataReceived(Data data) {
        if (StockServer_Debug) {
            System.err.println("StockServer: getName.");
        }

	timer = 0;
    }


    public void
    byteArrayDestroyed(SessionEvent event) {
        String name = event.getResourceName();

	if (StockServer_Debug) {
            System.err.println("StockServer: byteArrayDestroyed:" +
                                " event: " + event);
        }

	stocks.remove(name);
    }


    public static void
    main(String args[]) {
        StockServer stockServer = new StockServer(args);

        if (StockServer_Debug) {
            System.err.println("StockServer: main:" +
                                " args: " + args);
        }

	System.err.println("Setup and bound Stock server.");
        stockServer.run();
    }
}
