
/*
 * @(#)ChatServer.java	1.24 99/05/12
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.chat;

import com.sun.media.jsdt.*;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to make sure that the Shared Data classes have
 * all the required API in them.
 *
 * It's based on the chat demonstration which is part of the
 * west-coast rmi distribution.
 *
 * @version     1.5 - 12th May 1999
 * @author      Rich Burridge
 */

public class
ChatServer implements ChatDebugFlags {

    private static String sessionName = "ChatSession";


    public static void
    main(String args[]) {
        ChatClient client      = null;
	Session    chatSession = null;
	URLString  url         = null;
	String     sessionType = null;
	String     hostname    = null;
	int        hostport    = 0;

        if (ChatServer_Debug) {
            System.err.println("ChatServer: main.");
        }

        hostname    = getHost(args);
	hostport    = getPort(args);
	sessionType = getType(args);
	url         = URLString.createSessionURL(hostname, hostport, 
						 sessionType, sessionName);

	try {

/* Registry running?  Start it if it isn't. */

	    if (RegistryFactory.registryExists(sessionType) == false) {
		RegistryFactory.startRegistry(sessionType);
	    }

/* Create a session, [re]bind it and create a channel. */
 
	    client      = new ChatClient("Server");
	    chatSession = SessionFactory.createSession(client, url, false);
	    chatSession.createChannel(client, "ChatChannel", true, true, false);
            System.err.println("Setup and bound Chat server.");
        } catch (JSDTException e) {
            System.err.println("ChatServer: main: shared data exception: " + e);
            if (ChatServer_Debug) {
		e.printStackTrace();
	    }
        }
    }


    private static String
    getHost(String args[]) {
        String defHost = "stard";  /* Default host name for connections. */
        int length = args.length;

        if (ChatServer_Debug) {
            System.err.println("ChatServer: getHost.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-server")) {
                if (++i < length) {
                    return(args[i]);
                }
            }
        }
        return(defHost);
    }


    private static int
    getPort(String args[]) {
        int defPort = 4461;   /* Default port number for connections. */
        int length = args.length;

        if (ChatServer_Debug) {
            System.err.println("ChatServer: getPort.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-port")) {
                if (++i < length) {
                    return(Integer.parseInt(args[i]));
                }
            }    
        }
        return(defPort);
    }


    private static String
    getType(String args[]) {
        String defType = "socket";   /* Default Session type. */
        int length = args.length;

        if (ChatServer_Debug) {
            System.err.println("ChatServer: getType.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-type")) {
                if (++i < length) {
                    return(args[i]);
                }
            }    
        }
        return(defType);
    }
}
