
/*
 * @(#)CommandConsumer.java	1.15 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to test UDP channels.
 *
 * The server continuously sends a sound file over a UDP channel
 * to each client joined to the channel. These in turn, play the sound on
 * the workstation speaker.
 *
 * Code to implement this is based upon the audio jukebox work of Befhar
 * Razavi.
 *
 * @version     1.5 - 7th January 1999
 * @author      Rich Burridge
 */

import com.sun.media.jsdt.*;
import java.io.*;
import java.util.Hashtable;


public class
CommandConsumer implements ChannelConsumer, SoundDebugFlags {

    /** The name of this channel consumer. */
    protected String name;

    /** Hashtable of audio file names / sample rates. */
    private Hashtable sampleRates;

    /** Hashtable of audio file names / number of channels. */
    private Hashtable channels;

    /** Hashtable of audio file names / audio file encodings. */
    private Hashtable encodings;


    public
    CommandConsumer(String name) {
        if (CommandConsumer_Debug) {
            System.err.println("CommandConsumer: constructor:" +
			       " name: " + name);
        }

        this.name = name;
	sampleRates = new Hashtable();
        channels    = new Hashtable();
        encodings   = new Hashtable();
    }
 
 
    public synchronized void
    dataReceived(Data data) {
	ByteArrayInputStream bais;
        DataInputStream      dis;

        if (CommandConsumer_Debug) {
            System.err.println("CommandConsumer: dataReceived.");
        }

	bais = new ByteArrayInputStream(data.getDataAsBytes(), 
					0, data.getLength());
	dis  = new DataInputStream(bais);
	try {
	    String audioFileName = dis.readUTF();
	    int    sampleRate    = dis.readInt();
	    int    encoding      = dis.readInt();
	    int    noChannels    = dis.readInt();

	    if (CommandConsumer_Debug) {
		System.err.println("dataReceived:" +
				   " audio file name: " + audioFileName +
				   " sample rate: "     + sampleRate +
				   " encoding: "        + encoding +
				   " no of channels: "  + noChannels);
	    }

	    sampleRates.put(audioFileName, new Integer(sampleRate));
            encodings.put(audioFileName,   new Integer(encoding)); 
            channels.put(audioFileName,    new Integer(noChannels)); 
        } catch (IOException e) {
            if (CommandConsumer_Debug) {
                System.err.println("CommandConsumer: dataReceived:" +
                                   " exception " + e);
                e.printStackTrace();
            }
        }
    }


    public int
    getSampleRate(String audioFileName) {
        Integer sampleRate;

	if (CommandConsumer_Debug) {
            System.err.println("CommandConsumer: getSampleRate:" +
				" audio file name: " + audioFileName);
        }

        sampleRate = (Integer) sampleRates.get(audioFileName);
        return((sampleRate != null) ? sampleRate.intValue() : 0);
    }


    public int
    getChannels(String audioFileName) {
        Integer noChannels;

	if (CommandConsumer_Debug) {
            System.err.println("CommandConsumer: getChannels:" +
				" audio file name: " + audioFileName);
        }

        noChannels = (Integer) channels.get(audioFileName);
        return((noChannels != null) ? noChannels.intValue() : 0);
    }


    public int
    getEncoding(String audioFileName) {
        Integer encoding;

	if (CommandConsumer_Debug) {
            System.err.println("CommandConsumer: getEncoding:" +
				" audio file name: " + audioFileName);
        }

        encoding = (Integer) encodings.get(audioFileName);
	return((encoding != null) ? encoding.intValue() : 0);
    }



    public boolean
    isSoundInfo(String audioFileName) {
        if (CommandConsumer_Debug) {
            System.err.println("CommandConsumer: isSoundInfo:" +
				" audio file name: " + audioFileName);
        }

	return(sampleRates.containsKey(audioFileName) &&
	       channels.containsKey(audioFileName)    &&
	       encodings.containsKey(audioFileName));
    }
}
