 
/*
 * @(#)PlayAudioFile.java	1.12 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

import com.sun.media.jsdt.*;
 
/**
 * This is just a simple server/client based proof-of-concept
 * implementation to test UDP channels.
 *
 * The server continuously sends a sound file over a UDP channel
 * to each client joined to the channel. These in turn, play the sound on
 * the workstation speaker.
 *
 * Code to implement this is based upon the audio jukebox work of Befhar
 * Razavi.
 *
 * @version     1.5 - 7th January 1999
 * @author      Rich Burridge
 */
 
public class
PlayAudioFile extends Thread implements SoundDebugFlags {

    /** The channel to send the audio file over. */
    private Channel audioChannel = null;

    /** The server-side client already joined to this channel. */
    private Client client;

    /** The name of the audio file to play. */
    private String audioFileName = null;

    /** The audio file reader. */
    private AudioFileReader afr;


    public
    PlayAudioFile(Channel audioChannel, Client client,
		  String audioFileName) {
        if (PlayAudioFile_Debug) {
            System.err.println("PlayAudioFile: constructor:" +
                               " audio channel: "   + audioChannel +
			       " client: "          + client +
			       " audio file name: " + audioFileName);
        }

        this.audioChannel  = audioChannel;
	this.client        = client;
	this.audioFileName = audioFileName;
	try {
	    afr = new AudioFileReader(audioFileName, 50);
	} catch (Exception e) {
            System.err.println("PlayAudioFile: constructor: exception " + e);
            if (PlayAudioFile_Debug) {
                e.printStackTrace();
            }
	}
    }


    public AudioFileReader
    getReader() {
        if (PlayAudioFile_Debug) {
            System.err.println("PlayAudioFile: getReader.");
	}

	return(afr);
    }


/**
 * Grab a chunk of audio, send it, prefetch another chunk. If there is time
 * left until the next chunk is to be sent, go to sleep for that many
 * milliseconds. The audio can have variable size chunks.
  */

    public void
    run() {
	Data data;
	AudioClick click;
	int MSMARGIN = 5;
	long delay;

        if (PlayAudioFile_Debug) {
            System.err.println("PlayAudioFile: run.");
        }

	long tm = System.currentTimeMillis();
	click = (AudioClick) afr.get();
	while (true) {
	    if (click == null || click.length == 0) {

/* Reached end of file. Reset and server again. */

/* XXX: need to reset to beginning of the audio file correctly. */

		System.err.println("End of file reached.");
		break;
	    }

	    try {
                if (PlayAudioFile_Debug) {
                    System.err.println("PlayAudioFile: sending data:" +
				       " length: " + click.length);
                }
		data = new Data(click.buffer, click.length);
		audioChannel.sendToAll(client, data);
	    } catch (Exception e) {
                System.err.println("PlayAudioFile: run: exception " + e);
                if (PlayAudioFile_Debug) {
                    e.printStackTrace();
                }
	    }

/* Remember how much data was sent and delay accordingly. */

	    tm += click.length * 1000 / afr.getSampleRate();

/* Use the delay to prefetch the next audio click. */

	    click = (AudioClick) afr.get();
	    delay = tm - System.currentTimeMillis();

/* If the time to be delayed is less than a margin it is not worth the
 * overhead of sleeping.
 */

	    if (delay > MSMARGIN) {
		try {
		    Thread.sleep(delay);
		} catch (Exception e) {
                    System.err.println("PlayAudioFile: run: exception " + e);
                    if (PlayAudioFile_Debug) {
                        e.printStackTrace();
                    }
		}
	    } else {
		tm = System.currentTimeMillis();
	    }
	}
    }
}
