
/*
 * @(#)SoundCanvas.java	1.12 99/01/07
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

import com.sun.media.jsdt.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to test UDP channels.
 *
 * The server continuously sends a sound file over a UDP channel
 * to each client joined to the channel. These in turn, play the sound on
 * the workstation speaker.
 *
 * @version     1.5 - 7th January 1999
 * @author      Rich Burridge
 */

public class
SoundCanvas extends Canvas implements SoundDebugFlags {

    /** The image to display when no audio file is currently selected. */
    private Image jukebox;

    /** The artwork images for the various audio files. */
    private Image images[];

    /** The current image number that we are displaying (or -1 if none). */
    private int currentImageNo = -1;


    public
    SoundCanvas(Image jukebox, Image images[]) {
        if (SoundCanvas_Debug) {
            System.err.println("SoundCanvas: constructor:" +
                                " jukebox image: " + jukebox +
                                " images: "        + images);
        }

        this.jukebox = jukebox;
        this.images  = images;
    }


    public void
    setImageNo(int imageNo) {
        if (SoundCanvas_Debug) {
            System.err.println("SoundCanvas: setImageNo:" +
                                " image#: " + imageNo);
        }

	currentImageNo = imageNo;
    }


    public void
    paint(Graphics g) {
        Image image;
	int cWidth, cHeight, iWidth, iHeight, x, y;

	if (SoundCanvas_Debug) {
            System.err.println("SoundCanvas: paint:" +
				" graphics: " + g);
        }

        if (currentImageNo == -1) {
            image = jukebox;
        } else {
            image = images[currentImageNo];
        }

	cWidth  = getSize().width;
	cHeight = getSize().height;
	iWidth  = image.getWidth(null);
	iHeight = image.getHeight(null);
	x       = (cWidth - iWidth) / 2;
	y       = (cHeight - iHeight) / 2;

	g.drawImage(image, x, y, this);
    }
}
