
/*
 * @(#)SoundServer.java	1.25 99/05/12
 *
 * Copyright (c) 1993-1999 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You shall
 * not disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package examples.sound;

import com.sun.media.jsdt.*;

/**
 * This is just a simple server/client based proof-of-concept
 * implementation to test UDP channels.
 *
 * The server continuously sends a sound file over a UDP channel
 * to each client joined to the channel. These in turn, play the sound on
 * the workstation speaker.
 *
 * @version     1.5 - 12th May 1999
 * @author      Rich Burridge
 */

public class
SoundServer implements SoundDebugFlags {

    private static String sessionName        = "SoundSession";
    private static String commandChannelName = "CommandChannel";
    private static String tracks[] = { 
      "Keep Your Ears Open;images/keepopen.jpg;sounds/keepopen.au",
      "See Everything;images/ckeepno.jpg;sounds/ckeepno.au",
      "Java Here;images/javahere.jpg;sounds/javahere.au"
    };


    public static void
    main(String args[]) {
        Channel       audioChannels[] = null;
	Channel       commandChannel  = null;
	SoundClient   client          = null;
	SoundListener listeners[]     = null;
        Session       soundSession    = null;
        URLString     url             = null;
        String        sessionType     = null;
	String        hostname        = null;
        int           hostport        = 0;
	int           noTracks        = tracks.length;
 
        if (SoundServer_Debug) {
            System.err.println("SoundServer: main.");
        }
 
        hostname    = getHost(args);
        hostport    = getPort(args);
        sessionType = getType(args);
	url         = URLString.createSessionURL(hostname, hostport, 
						 sessionType, sessionName);
        try {
 
/* Registry running?  Start it if it isn't. */

            if (RegistryFactory.registryExists(sessionType) == false) {
                RegistryFactory.startRegistry(sessionType);
            }

/* Create a session, bind it, join it and create two channels. */

            client       = new SoundClient("Server");
	    soundSession = SessionFactory.createSession(client, url, true);

            commandChannel = soundSession.createChannel(client,
                                   commandChannelName, true, true, true);

	    audioChannels = new Channel[noTracks];
	    listeners     = new SoundListener[noTracks];

	    for (int i = 0; i < noTracks; i++) {
		String audioFileName = tracks[i].substring(
                                            tracks[i].lastIndexOf(';')+1);

		audioChannels[i] = soundSession.createChannel(client, 
				       tracks[i], false, false, true);
		listeners[i] = new SoundListener(audioFileName, client, 
					     audioChannels[i], commandChannel);
		audioChannels[i].addChannelListener(listeners[i]);
	    }

	    System.err.println("Setup and bound Sound server.");
        } catch (JSDTException e) {
            System.err.println("SoundServer: main: " +
			       "shared data exception: " + e);
            if (SoundServer_Debug) {
                e.printStackTrace();
            }
        }
    }    


    private static String
    getHost(String args[]) {
        String defHost = "stard";  /* Default host name for connections. */
        int length = args.length;

        if (SoundServer_Debug) {
            System.err.println("SoundServer: getHost.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-server")) {
                if (++i < length) {
                    return(args[i]);
                }
            }    
        }
        return(defHost);
    }


    private static int
    getPort(String args[]) {
        int defPort = 4461;   /* Default port number for connections. */
        int length = args.length;

        if (SoundServer_Debug) {
            System.err.println("SoundServer: getPort.");
        }

        for (int i = 0; i < length; i++) {
            if (args[i].equals("-port")) {
                if (++i < length) {
                    return(Integer.parseInt(args[i]));
                }
            }    
        }
        return(defPort);
    }


    private static String
    getType(String args[]) {
        String defType = "socket";   /* Default Session type. */
        int length = args.length;

        if (SoundServer_Debug) {
            System.err.println("SoundServer: getType.");
        }
 
        for (int i = 0; i < length; i++) {
            if (args[i].equals("-type")) {
                if (++i < length) {
                    return(args[i]);
                }
            }
        }
        return(defType);
    }
}
